/**

Copyright 2019 Forestry.io Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/

import * as React from 'react'
import { wrapFieldsWithMeta } from './wrapFieldWithMeta'
import { InputProps, ImageUpload } from '../components'
import { Media, MediaStore } from '@tinacms/core'
import { useCMS } from '@tinacms/react-core'
import { parse } from './textFormat'
import { useState, useEffect } from 'react'

interface ImageProps {
  path: string
  previewSrc?: MediaStore['previewSrc']
  uploadDir?(formValues: any): string
  clearable?: boolean
}

export function usePreviewSrc(
  value: string,
  fieldName: string,
  formValues: any,
  getPreviewSrc?: MediaStore['previewSrc']
): [string, boolean] {
  const cms = useCMS()
  const getSrc = getPreviewSrc || cms.media.previewSrc
  const [{ src, loading }, setState] = useState({
    src: '',
    loading: true,
  })

  useEffect(() => {
    let componentUnmounted = false
    let tmpSrc = ''
    ;(async () => {
      try {
        tmpSrc = await getSrc(value, fieldName, formValues)
      } catch {}

      if (componentUnmounted) return

      setState({
        src: tmpSrc,
        loading: false,
      })
    })()

    return () => {
      componentUnmounted = true
    }
  }, [value])

  return [src, loading]
}

export const ImageField = wrapFieldsWithMeta<InputProps, ImageProps>(props => {
  const cms = useCMS()
  const { form, field } = props
  const { name, value } = props.input
  const [src, srcIsLoading] = usePreviewSrc(
    value,
    name,
    form.getState().values,
    field.previewSrc
  )

  let onClear: any
  if (props.field.clearable) {
    onClear = () => props.input.onChange('')
  }

  function onChange(media?: Media) {
    if (media) {
      props.input.onChange('')
      props.input.onChange(media)
    }
  }

  const uploadDir = props.field.uploadDir || (() => '')

  return (
    <ImageUpload
      value={value}
      previewSrc={src}
      loading={srcIsLoading}
      onClick={() => {
        const directory = uploadDir(props.form.getState().values)
        cms.media.open({
          allowDelete: true,
          directory,
          onSelect: onChange,
        })
      }}
      onDrop={async ([file]: File[]) => {
        const directory = uploadDir(props.form.getState().values)

        const [media] = await cms.media.persist([
          {
            directory,
            file,
          },
        ])

        onChange(media)
      }}
      onClear={onClear}
    />
  )
})

export const ImageFieldPlugin = {
  name: 'image',
  Component: ImageField,
  parse,
}
